import { useEditor } from "@/modules/editor";
import { Button, Dropdown } from "ant-design-vue";
import { defineUI, queenApi } from "queenjs";
import { TipIcons } from "../../TipIcons";
import History from "../Toolbar/History";
import { ShareBox } from "./ShareBox";
import PreviewDesignModal from "./PreviewDesignModal";

export default defineUI({
  setup() {
    const editor = useEditor();
    const { store, actions, controls, jumpIndexHtml } = editor;

    function showPreviewModal() {
      controls.screenCtrl.saveScreenPage();
      const rootPage = controls.pageCtrl.rootPage;
      rootPage.value.pageMode = controls.screenCtrl.state.screen.pageMode;
      rootPage.value.pageSizeType =
        controls.screenCtrl.state.screen.pageSizeType;

      queenApi.dialog(
        <PreviewDesignModal data={controls.pageCtrl.designData} />,
        {
          fullscreen: true,
          closable: false,
        }
      );
    }

    return () => (
      <div class="relative flex justify-between items-center">
        <div class="flex items-center">
          <aside>
            <img
              class="h-40px cursor-pointer"
              src={require("@/assets/imgs/Logo.png")}
              alt="logo"
              onClick={() => jumpIndexHtml()}
            />
          </aside>
          <History class="ml-100px" />
        </div>
        {/* <Radio.Group
          value={store.mode}
          onChange={(e) => actions.switchMode(e.target.value)}
        >
          <Radio.Button value={store.isEditPage ? "editPage" : "editComp"}>
            编辑
          </Radio.Button>
          <Radio.Button value="preview">预览</Radio.Button>
        </Radio.Group> */}
        <div class="absolute left-1/2 top-1/2 transform -translate-x-1/2 -translate-y-1/2">
          <div class="text-16px font-bold text-light-50 max-w-240px truncate leading-normal">
            {controls.pageCtrl.state.title}
          </div>
        </div>
        <aside class="space-x-16px flex items-center">
          <TipIcons.Preview
            class="text-20px text-light-50 bg-dark-50 p-6px transform rotate-90 rounded cursor-pointer transition hover:(!bg-dark-100)"
            onClick={showPreviewModal}
          />
          {store.isEditPage && (
            <Dropdown
              overlay={<ShareBox />}
              trigger="click"
              placement="bottomRight"
            >
              <Button
                type="default"
                class="text-12px bg-dark-50 hover:(bg-dark-100 border-transparent text-light-50)"
              >
                分享
              </Button>
            </Dropdown>
          )}
          <Button
            class="text-12px text-dark-500 hover:text-dark-500"
            type="primary"
            onClick={() => actions.saveDesign()}
          >
            保存
          </Button>
        </aside>
      </div>
    );
  },
});
