import List from "@/components/AssetsList";
import { PromotionController } from "@/modules/resource/controllers/PromotionController";
import { defineUI } from "queenjs";
import { onMounted } from "vue";
import { any } from "vue-types";
import Header from "./Header";
import Filter from "./CategoryFilter";
import PromotionItem from "./PromotionItem";
import { useAuth } from "@queenjs-modules/auth";

export default defineUI({
  props: {
    Controller: any<PromotionController>().isRequired,
  },

  slots: {
    Header,
    Filter,
    List,
  },
  setup(props, { slots }) {
    const auth = useAuth();

    const FilterChange = (query: any) => {
      const ctrlQuery = props.Controller.ListCtrl.state.query || {};
      props.Controller.ListCtrl.state.query = {
        ...ctrlQuery,
        categories: query,
      };
      props.Controller.ListCtrl.loadPage(1);
    };
    onMounted(() => {
      props.Controller.ListCtrl.state.query = {};
      props.Controller.ListCtrl.loadPage(1);
    });

    return () => {
      //@ts-ignore
      const isSys = (auth.store.userInfo.roles || []).indexOf("system") > -1;
      return (
        <div>
          <slots.Header onAdd={props.Controller.createPromotion} />
          <slots.Filter
            sourceType="template"
            onAdd={props.Controller.createPromotion}
            onChange={FilterChange}
          />
          <slots.List
            gap="25px"
            class="my-30px"
            columns={5}
            control={props.Controller.ListCtrl}
            item={(record: any) => (
              <PromotionItem
                record={record}
                onMenu={(name) => {
                  props.Controller.onMenuClick(name, record, isSys);
                }}
                onEdit={(record) => props.Controller.onEdit(record)}
              />
            )}
          />
        </div>
      );
    };
  },
});
